var _ = require("lodash");
var assert = require("assert");
var techRuuid = require("node-tech-ruuid");
var logger = require("node-tech-logger");

module.exports = function(options) {

    var reporters = options.reporters,
        owner = options.owner;

    assert.ok(reporters, "reporter required");
    assert.ok(owner, "owner required");

    var logs = {};

    function getLog(ruuid) {
        if (!logs[ruuid]) {
            logs[ruuid] = {
                items: []
            };
        }
        return logs[ruuid];
    }

    function gatherFact(ruuid, duration) {
        var attributes = logAttributesAsMap(ruuid, duration);
        return {
            attributes : mapAsNameValueArray(attributes)
        };
    }

    function logAttributesAsMap(ruuid, duration) {
        var attributes = {
            "tx": ruuid,
            "duration": duration.ms
        };
        var log = getLog(ruuid);
        return _.reduce(log.items, function(memo, item) {

            logger.debug("Recieved log item", item);

            var varNameBase = item.type + "_" + item.category.toLowerCase();
            var countVarName = "nb_" + varNameBase;
            var durationVarName = "dur_" + varNameBase;

            if (!memo[countVarName]) {
                memo[countVarName] = 0;
            }
            if (!memo[durationVarName]) {
                memo[durationVarName] = 0;
            }

            logger.debug("Will use count and duration var", countVarName, durationVarName);

            memo[countVarName] = memo[countVarName] + 1;

            memo[durationVarName] = memo[durationVarName] + item.duration.ms;

            return memo;
        }, attributes);

    }

    function mapAsNameValueArray(map) {
        return _.reduce(map, function(memo, value, key) {
            memo.push({
                name: key,
                value: value
            });
            return memo;
        }, []);
    }

    /**
     * Listen for perfLog events.
     *
     * @param emitter
     *        {EventEmitter}
     * @param eventName
     *        {String} name of a perf log event. Should have a ruuid.
     * @param getType
     *        {Function} get the type of the event (needed to organise them to build facts.)
     */
    function addListener(emitter, eventName, getType) {
        emitter.on(eventName, function(event) {
            assert.ok(event.ruuid, "Emitted perf event should have a ruuid !!");
            var log = getLog(event.ruuid);
            var type = getType(event);
            log.items.push(_.extend({
                type: type
            }, event));
        });
    }

    return {
        addListener: addListener,
        startRequest: function(ruuid, query, method) {
            techRuuid.check(ruuid);
            assert.ok(query, "Query is mandatory");
            assert.ok(method, "Method should be provided");
            var log = getLog(ruuid);
            log.query = query;
            log.method = method;
        },
        gatherFact: gatherFact,
        endRequest: function(ruuid, duration) {
            var fact = gatherFact(ruuid, duration);
            delete logs[ruuid];
            _.each(reporters, function(reporter) {
                reporter.reportFact(fact);
            });
        }
    };

};
